'use strict';

import { ReanimatedError } from "../../errors.js";
import { mockedRequestAnimationFrame } from "../../mockedRequestAnimationFrame.js";
import { isJest } from "../../PlatformChecker.js";
export function createJSWorkletsModule() {
  return new JSWorklets();
}

// In Node.js environments (like when static rendering with Expo Router)
// requestAnimationFrame is unavailable, so we use our mock.
// It also has to be mocked for Jest purposes (see `initializeUIRuntime`).
const requestAnimationFrameImpl = isJest() || !globalThis.requestAnimationFrame ? mockedRequestAnimationFrame : globalThis.requestAnimationFrame;
class JSWorklets {
  makeShareableClone() {
    throw new ReanimatedError('makeShareableClone should never be called in JSWorklets.');
  }
  scheduleOnUI(worklet) {
    // @ts-ignore web implementation has still not been updated after the rewrite,
    // this will be addressed once the web implementation updates are ready
    requestAnimationFrameImpl(worklet);
  }
  executeOnUIRuntimeSync(_shareable) {
    throw new ReanimatedError('`executeOnUIRuntimeSync` is not available in JSReanimated.');
  }
  createWorkletRuntime(_name, _initializer) {
    throw new ReanimatedError('createWorkletRuntime is not available in JSReanimated.');
  }
  scheduleOnRuntime() {
    throw new ReanimatedError('scheduleOnRuntime is not available in JSReanimated.');
  }
}
//# sourceMappingURL=JSWorklets.js.map